package com.gic.haoban.manage.service.service.hm.impl;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeListQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeQDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.dao.mapper.hm.TabHaobanHmQrcodeMapper;
import com.gic.haoban.manage.service.entity.hm.TabHaobanHmQrcode;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeClerkBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeGroupNumBO;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 活码(TabHaobanHmQrcode)表服务实现类
 *
 * @author mozhu
 * @since 2022-07-04 15:42:20
 */
@Service
public class HmQrcodeServiceImpl implements HmQrcodeService {

    private static org.apache.logging.log4j.Logger log = LogManager.getLogger(HmQrcodeService.class);
    @Autowired
    private TabHaobanHmQrcodeMapper tabHaobanHmQrcodeMapper;
    @Autowired
    private StaffApiService staffApiService;

    @Override
    public HmQrcodeBO queryById(Long hmId) {
        return EntityUtil.changeEntity(HmQrcodeBO.class, this.tabHaobanHmQrcodeMapper.queryById(hmId));
    }

    @Override
    public Page<HmQrcodeBO> queryByPage(HmQrcodeListQDTO hmQrcodeListQDTO) {
        PageHelper.startPage(hmQrcodeListQDTO.getPageNum(), hmQrcodeListQDTO.getPageSize());
        List<TabHaobanHmQrcode> tabHaobanHmQrcodes = tabHaobanHmQrcodeMapper.queryListByPage(hmQrcodeListQDTO);
        return PageUtil.changePageHelperToCurrentPage(new PageInfo<>(tabHaobanHmQrcodes), HmQrcodeBO.class);
    }

    @Override
    public int insert(HmQrcodeQDTO hmQrcodeQDTO) {
        hmQrcodeQDTO.setCreateTime(new Date());
        hmQrcodeQDTO.setUpdateTime(new Date());
        hmQrcodeQDTO.setPassFlag(1);
        hmQrcodeQDTO.setOverFlag(0);
        hmQrcodeQDTO.setAddNum(0);
        hmQrcodeQDTO.setStatusFlag(1);
        return this.tabHaobanHmQrcodeMapper.insert(EntityUtil.changeEntity(TabHaobanHmQrcode.class, hmQrcodeQDTO));
    }

    @Override
    public int update(HmQrcodeQDTO hmQrcodeQDTO) {
        hmQrcodeQDTO.setUpdateTime(new Date());
        return this.tabHaobanHmQrcodeMapper.update(EntityUtil.changeEntity(TabHaobanHmQrcode.class, hmQrcodeQDTO));
    }

    /**
     * 通过主键删除数据
     *
     * @param hmId 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Long hmId) {
        return this.tabHaobanHmQrcodeMapper.updateStatusById(2, hmId) > 0;
    }

    @Override
    public List<HmQrcodeBO> listByClerkId(String clerkId, String enterpriseId) {
        return EntityUtil.changeEntityListNew(HmQrcodeBO.class,
                tabHaobanHmQrcodeMapper.listByClerkId(clerkId, enterpriseId));
    }

    @Override
    public List<HmQrcodeBO> listByStoreId(String wxEnterpriseId, String storeId, Integer overFlag) {
        return EntityUtil.changeEntityListNew(HmQrcodeBO.class,
                tabHaobanHmQrcodeMapper.listByStoreId(wxEnterpriseId, storeId, overFlag));
    }

    @Override
    public int updateAddNumById(Long hmId, Integer overFlag) {
        return tabHaobanHmQrcodeMapper.updateAddNumById(hmId, overFlag);
    }

    @Override
    public void resetAddNumAndOverFlag() {
        tabHaobanHmQrcodeMapper.resetAddNumAndOverFlag();
    }

    @Override
    public HmQrcodeBO queryByClerkId(String clerkId, String wxEnterpriseId) {
        return EntityUtil.changeEntity(HmQrcodeBO.class,
                tabHaobanHmQrcodeMapper.queryByClerkId(clerkId, wxEnterpriseId));
    }

    @Override
    public HmQrcodeBO queryByClerkId(String clerkId, String wxEnterpriseId, Integer overFlag) {
        return EntityUtil.changeEntity(HmQrcodeBO.class,
                tabHaobanHmQrcodeMapper.queryByClerkIdAndOverFlag(clerkId, wxEnterpriseId, overFlag));
    }

    @Override
    public int updateStatusById(Long hmId, Integer statusFlag) {
        return tabHaobanHmQrcodeMapper.updateStatusById(statusFlag, hmId);
    }

    @Override
    public List<HmQrcodeBO> getHmStoreByEnterpriseId(String enterpriseId) {
        return EntityUtil.changeEntityListNew(HmQrcodeBO.class,
                tabHaobanHmQrcodeMapper.getHmStoreByEnterpriseId(enterpriseId));
    }

    @Override
    public List<HmQrcodeGroupNumBO> getGroupIdNumByEnterpriseId(String enterpriseId, List<Long> groupIds) {
        return tabHaobanHmQrcodeMapper.getGroupIdNumByEnterpriseId(enterpriseId, groupIds);
    }

    @Override
    public int getTotalByEnterpriseId(String enterpriseId) {
        return tabHaobanHmQrcodeMapper.getTotalByEnterpriseId(enterpriseId);
    }

    @Override
    public int batchUpdateQrcodeGroup(List<Long> hmIds, Long groupId) {
        return tabHaobanHmQrcodeMapper.batchUpdateQrCodeGroup(hmIds, groupId);
    }

    @Override
    public int getCanAddCount(String wxEnterpriseId, String enterprsieId, String clerkId) {
        // 如果有权限控制,进行管辖门店过0滤
        List<String> authStoreIdList = this.staffApiService.getHaoBanStoreIdsRolesByClerkId(clerkId, wxEnterpriseId);
        if (CollectionUtils.isEmpty(authStoreIdList)) {
            return 0;
        }
        if (!(authStoreIdList.size() == 1 && authStoreIdList.contains("-1"))) {
            log.info("部分门店={}", authStoreIdList.size());
        } else {
            log.info("全部门店={}", clerkId);
            authStoreIdList = null;
        }
        int count = this.tabHaobanHmQrcodeMapper.getCanAddCount(wxEnterpriseId, enterprsieId, authStoreIdList);
        return count;
    }

    @Override
    public List<HmQrcodeClerkBO> listCanAddClerk(String wxEnterpriseId, String enterprsieId, String clerkId) {
        List<String> authStoreIdList = this.staffApiService.getHaoBanStoreIdsRolesByClerkId(clerkId, wxEnterpriseId);
        if (CollectionUtils.isEmpty(authStoreIdList)) {
            return null;
        }
        if (!(authStoreIdList.size() == 1 && authStoreIdList.contains("-1"))) {
            log.info("全部门店={}", clerkId);
        } else {
            authStoreIdList = null;
        }
        List<HmQrcodeClerkBO> list = this.tabHaobanHmQrcodeMapper.listCanAddClerk(wxEnterpriseId, enterprsieId,
                authStoreIdList);
        return list;
    }

}
