package com.gic.haoban.manage.service.service.out.impl.hm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.redisson.api.RRateLimiter;
import org.redisson.api.RateIntervalUnit;
import org.redisson.api.RateType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.haoban.common.utils.DingUtils;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreSettingDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkWxaDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkSearchQDTO;
import com.gic.haoban.manage.api.service.hm.HmLinkApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanExternalClerkRelatedMapper;
import com.gic.haoban.manage.service.entity.TabHaobanExternalClerkRelated;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.pojo.bo.hm.HmLinkBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.pojo.bo.hm.KeyDataLinkBO;
import com.gic.haoban.manage.service.pojo.qo.HmQrcodeTempQO;
import com.gic.haoban.manage.service.service.KeyDataService;
import com.gic.haoban.manage.service.service.MemberQueryService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.hm.HmLinkService;
import com.gic.haoban.manage.service.service.hm.HmLinkStoreService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeTempService;
import com.gic.member.api.dto.MemberClerkDTO;
import com.gic.member.api.dto.es.MemberDataDTO;
import com.gic.member.api.dto.es.MemberStoreClerkDataDTO;
import com.gic.member.api.service.MemberClerkApiService;
import com.gic.member.api.service.MemberService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactDTO;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactResultDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

@Service("hmLinkApiService")
public class HmLinkApiServiceImpl implements HmLinkApiService {

    private static org.apache.logging.log4j.Logger log = LogManager.getLogger(HmLinkApiService.class);

    @Autowired
    private HmLinkService hmLinkService;
    @Autowired
    private HmLinkStoreService hmLinkStoreService;
    @Autowired
    private KeyDataService keyDataService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private MemberQueryService memberQueryService;
    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private Config config;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private HmQrcodeTempService hmQrcodeTempService;
    @Autowired
    private TabHaobanExternalClerkRelatedMapper tabHaobanExternalClerkRelatedMapper;
    @Autowired
    private MemberClerkApiService memberClerkApiService;

    @Override
    public ServiceResponse<String> getHmLinkUrl() {
        return ServiceResponse.success(config.getHmLinkUrl());
    }

    @Override
    public ServiceResponse<HmLinkDTO> getByLinkId(String wxEnterpriseId, String enterpriseId, Long linkId) {
        HmLinkBO link = this.hmLinkService.getByLinkId(wxEnterpriseId, enterpriseId, linkId);
        if (null != link) {
            HmLinkDTO dto = EntityUtil.changeEntityByJSON(HmLinkDTO.class, link);
            List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(dto.getWxEnterpriseId(),
                    dto.getEnterpriseId(), linkId);
            dto.setStoreList(storeList);
            return ServiceResponse.success(dto);
        }
        return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接不存在");
    }

    @Override
    public ServiceResponse<HmLinkDTO> getByShortCode(String shortCode) {
        HmLinkBO bo = this.hmLinkService.getByLinkShortCode(shortCode);
        if (null == bo) {
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接不存在");
        }
        HmLinkDTO dto = EntityUtil.changeEntityByJSON(HmLinkDTO.class, bo);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<List<HmLinkDTO>> listForSms(String wxEnterpriseId, String enterprieId, String linkName,
            Integer linkType) {
        List<HmLinkBO> list = this.hmLinkService.listForSms(wxEnterpriseId, enterprieId, linkName, linkType);
        List<HmLinkDTO> retList = EntityUtil.changeEntityListByJSON(HmLinkDTO.class, list);
        return ServiceResponse.success(retList);
    }

    @Override
    public ServiceResponse<String> save(HmLinkDTO dto) {
        String id = this.hmLinkService.saveLink(dto, dto.getStoreList());
        return ServiceResponse.success(id);
    }

    @Override
    public ServiceResponse<Page<HmLinkDTO>> listPage(HmLinkSearchQDTO qdto, BasePageInfo basePageInfo) {
        Page<HmLinkBO> page = this.hmLinkService.listPage(qdto, basePageInfo);
        Page<HmLinkDTO> resultPage = PageHelperUtils.changePageToCurrentPage(page, HmLinkDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<HmLinkWxaDTO> getLinkHmFromWxa(String enterpriseId, Integer whitchStore, String inStoreId,
            String inClerkId, String linkShortCode, String mixPhone, String unionid, String memberId) {
        if ("-1".equals(memberId)) {
            memberId = null;
        }
        HmQrcodeBO hm = null;
        HmLinkWxaDTO retDTO = new HmLinkWxaDTO();
        log.info("小程序获取活码={},linkShortCode={},mixPhone={},memberId={},unionid={},门店={}{}", enterpriseId, linkShortCode,
                mixPhone, memberId, unionid,inStoreId , whitchStore);
        HmLinkBO link = this.hmLinkService.getByLinkShortCodeCache(linkShortCode);
        if (null == link) {
            log.error("链接记录不存,linkShortCode={}", linkShortCode);
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接记录不存");
        }
        Long linkId = link.getLinkId();
        String wxEnterpriseId = link.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            log.error("未查询到微信企业,id={}", wxEnterpriseId);
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "未查询到微信企业");
        }
        // 判断导购是否有活码,如果没有返回错误
        if (StringUtils.isNotBlank(inClerkId)) {
            hm = this.getHmFromClerk(inClerkId, wxEnterpriseId);
            if (null == hm) {
                return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "导购活码不存在");
            }
        }
        // 直接返回门店的导购活码,如果没有返回错误
        if (null == hm && StringUtils.isNotBlank(inStoreId)) {
            String customRuleJson = link.getCustomRuleJson();
            log.info("自定义规则={}", customRuleJson);
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                int dz = 0, dy = 0;
                // 1推荐门店 2附件门店 3省市区门店
                int storeType = storeRuleObj.getIntValue("store_type");
                // 省市区
                if (whitchStore == 3) {
                    HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
                    dz = cityStore.getDz();
                    dy = cityStore.getDy();
                }
                if (whitchStore == 1 || whitchStore == 2) {
                    HmLinkStoreSettingDTO store = storeRuleObj.getObject("store", HmLinkStoreSettingDTO.class);
                    if (whitchStore == 2 || (storeType == 1 && whitchStore == 1)) {
                        // 附近门店 或者 推荐门店
                        dz = store.getDz();
                        dy = store.getDy();
                    } else if (storeType == 2 && whitchStore == 1) {
                        // 附近门店的推荐门店
                        dz = store.getNdz();
                        dy = store.getNdy();
                    }
                }
                hm = this.getHmFormStore(wxEnterpriseId, inStoreId, dz, dy);
                log.info("获取到的门店配置dz={},dy={},获取的活码={}", dz, dy, JSON.toJSONString(hm));
                if (null == hm) {
                    return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "导购活码不存在");
                }
                retDTO.setSm("会员选门店活码");
            }
        }
        // 会员门店
        if (null == hm) {
            HmLinkStoreSettingDTO fwStore = null;
            HmLinkStoreSettingDTO xgStore = null;
            if (StringUtils.isNotBlank(link.getStoreRuleJson())) {
                String storeRuleJson = link.getStoreRuleJson();
                log.info("会员门店规则={}", storeRuleJson);
                JSONObject storeRuleObj = JSONObject.parseObject(storeRuleJson);
                fwStore = storeRuleObj.getObject("fw_store", HmLinkStoreSettingDTO.class);
                if (null == fwStore || fwStore.getOpen() == 0) {
                    fwStore = null;
                }
                xgStore = storeRuleObj.getObject("xg_store", HmLinkStoreSettingDTO.class);
                if (null == xgStore || xgStore.getOpen() == 0) {
                    xgStore = null;
                }
                if (StringUtils.isNotEmpty(mixPhone)) {
                    String mobile = this.keyDataService.getMobile(mixPhone, linkShortCode);
                    log.info("小程序获取活码mobile={},mixPhone={}", mobile, mixPhone);
                    String mobileMemberId = this.memberService.getMemberIdByPhoneNumber(mobile, enterpriseId);
                    if (StringUtils.isNotBlank(mobileMemberId)) {
                        memberId = mobileMemberId;
                    }
                }
                log.info("小程序获取活码memberId={}", memberId);
                if (StringUtils.isNotBlank(memberId)) {
                    MemberDataDTO member = this.getMemberClerk(enterpriseId, memberId);
                    String mainClerkId = member.getMainClerkId();
                    String mainStoreId = member.getMainStoreId();
                    // 服务门店
                    if (null != fwStore) {
                        if (fwStore.getZs() == 1 && StringUtils.isNotBlank(mainClerkId)) {
                            hm = this.getHmFromClerk(mainClerkId, wxEnterpriseId);
                        }
                        if (null == hm && StringUtils.isNotBlank(mainStoreId)) {
                            hm = this.getHmFormStore(wxEnterpriseId, mainStoreId, fwStore.getDz(), fwStore.getDy());
                        }
                    }
                    if(null != hm) {
                        retDTO.setSm("会员主门店活码");
                    }
                    // 协管门店
                    List<MemberStoreClerkDataDTO> list = member.getStoreClerkRel();
                    if (null == hm && null != xgStore && CollectionUtils.isNotEmpty(list)) {
                        // 排序协管导购
                        list = this.sortMemberClerk(enterpriseId, memberId, list);
                        List<String> storeIdList = new ArrayList<>();
                        // 协管导购
                        for (MemberStoreClerkDataDTO item : list) {
                            String clerkId = item.getClerkIdRel();
                            if (clerkId.equals(mainClerkId)) {
                                continue;
                            }
                            hm = this.getHmFromClerk(clerkId, wxEnterpriseId);
                            if (null != hm) {
                                storeIdList = null;
                                break;
                            }
                            storeIdList.add(item.getStoreIdRel());
                        }
                        // 店长
                        if (null == hm && xgStore.getDz() == 1 && CollectionUtils.isNotEmpty(storeIdList)) {
                            for (String storeId : storeIdList) {
                                hm = this.getHmFormStore(wxEnterpriseId, storeId, 1, 0);
                                if (null != hm) {
                                    break;
                                }
                            }
                        }
                        // 店员
                        if (null == hm && xgStore.getDy() == 1 && CollectionUtils.isNotEmpty(storeIdList)) {
                            for (String storeId : storeIdList) {
                                hm = this.getHmFormStore(wxEnterpriseId, storeId, 0, 1);
                                if (null != hm) {
                                    break;
                                }
                            }
                        }
                    }
                    if(null != hm) {
                        retDTO.setSm("会员协管活码");
                    }
                }
            }
        }
        if (null == hm && StringUtils.isNotBlank(link.getCustomRuleJson())) {
            String customRuleJson = link.getCustomRuleJson();
            log.info("自定义规则={}", customRuleJson);
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                // 门店规则1推荐门店 2附件门店
                int storeType = storeRuleObj.getIntValue("store_type");
                // 活码的门店
                List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(wxEnterpriseId, enterpriseId,
                        linkId);
                log.info("门店规则={},推荐门店列表={}", storeType, JSON.toJSONString(storeList));
                HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
                // 是否开启省市区门店查询
                int selectStoreFlag = cityStore.getOpen();
                log.info("省市区开启={}", selectStoreFlag);
                retDTO.setStoreType(storeType);
                retDTO.setStoreList(storeList);
                retDTO.setSelectStoreFlag(selectStoreFlag);
            }
        }
        if (null != hm) {
            log.info("获取的单人活码={}",JSON.toJSONString(hm));
            // 判断是否已经是好友
            boolean isFriend = false;
            if (null != memberId) {
                String staffId = hm.getStaffId();
                TabHaobanExternalClerkRelated friend = tabHaobanExternalClerkRelatedMapper
                        .getByMemberIdAndStaffId(staffId, enterpriseId, memberId);
                if (null != friend) {
                    isFriend = true;
                    log.info("已经是好友,staffid={},memberId={}", staffId, memberId);
                    retDTO.setHmQrcode(hm.getWxQrcode());
                }
            }

            if (!isFriend) {
                KeyDataLinkBO bo = new KeyDataLinkBO();
                bo.setUnionid(unionid);
                bo.setLinkId(linkId);
                bo.setHmId(hm.getHmId());
                String dthmKey = this.keyDataService.saveDataForLink(enterpriseId, bo);
                if (null != dthmKey) {
                    ServiceResponse<String> addResp = this.addQrcode(hm, dthmKey, wxEnterpriseDTO.getCorpid(), retDTO);
                    if (!addResp.isSuccess()) {
                        retDTO.setHmQrcode(hm.getWxQrcode());
                    } else {
                        retDTO.setHmQrcode(addResp.getResult());
                    }
                }
            }
            // 查询页面
            retDTO.setPageId(link.getPageId());
        }
        // 生成动态活码
        log.info(JSON.toJSONString(retDTO, true));
        return ServiceResponse.success(retDTO);
    }

    private List<MemberStoreClerkDataDTO> sortMemberClerk(String enterpriseId, String memberId,
            List<MemberStoreClerkDataDTO> list) {
        ServiceResponse<List<MemberClerkDTO>> resp = this.memberClerkApiService.getMemberMainClerkList(enterpriseId,
                memberId);
        if (!resp.isSuccess() && CollectionUtils.isEmpty(resp.getResult())) {
            log.info("member主导购={},memberId", resp.getMessage(), memberId);
            return list;
        }
        List<MemberStoreClerkDataDTO> retList = new ArrayList<>();
        List<String> mainClerkIdList = resp.getResult().stream().map(dto -> dto.getClerkId())
                .collect(Collectors.toList());
        List<MemberStoreClerkDataDTO> notMainClerkList = new ArrayList<>();
        for (MemberStoreClerkDataDTO item : retList) {
            if (mainClerkIdList.contains(item.getClerkIdRel())) {
                retList.add(item);
            } else {
                notMainClerkList.add(item);
            }
        }
        retList.addAll(notMainClerkList);
        return retList;
    }

    // 创建活码
    private ServiceResponse<String> addQrcode(HmQrcodeBO hm, String key, String corpid, HmLinkWxaDTO retDTO) {
        String limitQrcode = this.limitIp(hm);
        if (StringUtils.isNotEmpty(limitQrcode)) {
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "超限制数");
        }
        QywxExternalcontactDTO paramsDTO = new QywxExternalcontactDTO();
        // 联系方式类型,1-单人, 2-多人 场景，1-在小程序中联系，2-通过二维码联系
        paramsDTO.setType(1);
        paramsDTO.setScene(2);
        paramsDTO.setRemark(hm.getName());
        paramsDTO.setSkipVerify(hm.getPassFlag() == 1);
        paramsDTO.setState("DT" + key);
        paramsDTO.setUser(Arrays.asList(hm.getWxUserId()));
        QywxExternalcontactResultDTO wxResp = qywxUserApiService.addContactWay(corpid, config.getWxSuiteid(),
                paramsDTO);
        log.info("创建动态活码参数corpid={},app={},param={},返回={}", corpid, config.getWxSuiteid(), JSON.toJSONString(paramsDTO),
                JSON.toJSONString(wxResp));
        if (null != wxResp && 0 == wxResp.getErrcode()) {
            retDTO.setHmQrcode(wxResp.getQr_code());
            HmQrcodeTempQO qo = new HmQrcodeTempQO();
            qo.setWxEnterpriseId(hm.getWxEnterpriseId());
            qo.setWxConfigId(wxResp.getConfig_id());
            qo.setWxQrcode(wxResp.getQr_code());
            qo.setRelationId(key);
            qo.setCorpid(corpid);
            this.hmQrcodeTempService.save(qo);
            return ServiceResponse.success(wxResp.getQr_code());
        }
        return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "企微错误：" + wxResp.getErrmsg());
    }

    private String limitIp(HmQrcodeBO hm) {
        String ipKey = "hmadd:limit:ip";
        int ipLimitCount = 1;
        RRateLimiter rateLimiter = RedisUtil.getRedisClient().getRateLimiter(ipKey);
        rateLimiter.trySetRate(RateType.OVERALL, ipLimitCount, 1, RateIntervalUnit.SECONDS);
        if (rateLimiter.tryAcquire()) {
            return null;
        }
        String msg = "活码调用次数告警【ip】";
        DingUtils.send(msg, false);
        return hm.getWxQrcode();
    }

    private HmQrcodeBO getHmFromClerk(String clerkId, String wxEnterpriseId) {
        if (StringUtils.isBlank(clerkId) || "-1".equals(clerkId)) {
            return null;
        }
        HmQrcodeBO hm = this.hmQrcodeService.queryByClerkId(clerkId, wxEnterpriseId, 0);
        return hm;
    }

    private HmQrcodeBO getHmFormStore(String wxEnterpriseId, String inStoreId, int dz, int dy) {
        if (StringUtils.isBlank(inStoreId) || "-1".equals(inStoreId)) {
            return null;
        }
        HmQrcodeBO hm = null;
        if (dz == 1) {
            ClerkDTO dzClerk = this.clerkService.getClerkLeaderByStoreId(inStoreId);
            if (null != dzClerk) {
                hm = this.getHmFromClerk(dzClerk.getClerkId(), wxEnterpriseId);
            } else {
                log.info("店长不存在,storeId={}", inStoreId);
            }
        }
        if (null == hm && dy == 1) {
            List<HmQrcodeBO> list = this.hmQrcodeService.listByStoreId(wxEnterpriseId, inStoreId, 0);
            if (CollectionUtils.isNotEmpty(list)) {
                hm = list.get(new Random().nextInt(list.size()));
            }
        }
        return hm;
    }

    // 冗余会员信息
    private MemberDataDTO getMemberClerk(String enterpriseId, String memberId) {
        String returnFields = "memberId,mainClerkId,mainStoreId,storeClerkRel";
        JSONObject memberObj = this.memberQueryService.queryDataSingle(enterpriseId, memberId, returnFields);
        if (null == memberObj) {
            return null;
        }
        log.info("会员信息={}", memberObj.toJSONString());
        MemberDataDTO memberDataDTO = memberObj.toJavaObject(MemberDataDTO.class);
        return memberDataDTO;
    }
}
